@@ -25,11 +25,13 @@ class DataloaderUnitTest(unittest.TestCase):
         #  every test class must be present for training
         self.assertTrue(set(ytrain) >= set(ytest))
 
-    def _loader_sanity_check(
-            self,
-            xtrain, ytrain,
-            xval=None, yval=None,
-            xtest=None, ytest=None):
+    def _loader_sanity_check(self,
+                             xtrain,
+                             ytrain,
+                             xval=None,
+                             yval=None,
+                             xtest=None,
+                             ytest=None):
         self._check_pair(xtrain, ytrain)
 
         if xtest is not None:
