@@ -1,17 +1,41 @@
 """Module containing the evaluator base class and some default evaluators."""
+import datetime
 import json
+import os
+import pickle
 import warnings
 
 import numpy as np
 import pandas as pd
-from sklearn import metrics
 from sklearn.base import clone
+from sklearn import metrics
 from sklearn.model_selection import BaseCrossValidator
 from sklearn.model_selection import GridSearchCV
 from sklearn.model_selection import PredefinedSplit
 from sklearn.utils.validation import indexable
 
 from .base import Evaluator
+from .utils import LOGGER
+
+
+def _check_fixed_split_data_shape(data):
+    assert (len(data) == 2) or (len(data) == 3)
+    assert len(data[0]) == 2
+
+    if len(data) == 2:
+        assert len(data[1]) == 2
+    else:
+        assert len(data[2]) == 2
+
+
+def _extract_train_test_data(data):
+    _check_fixed_split_data_shape(data)
+
+    if len(data) == 2:
+        return data
+    else:
+        train, _, test = data
+        return train, test
 
 
 class ModelCallbackWrapper(Evaluator):
@@ -32,13 +56,17 @@ class ModelCallbackWrapper(Evaluator):
                 f'the wrapped evaluator {wrapped_evaluator.__class__.__name__}'
                 'does not have a fitted_model attribute.')
         self.wrapped_evaluator = wrapped_evaluator
-        self.callback = callback
+        if not type(callback) == list:
+            self.callbacks = [callback]
+        else:
+            self.callbacks = callback
         self.callback_args = callback_args
         self.callback_kwargs = callback_kwargs
+        self.callback_results = {}
 
     def evaluate(self, model, data):
         """
-        Evaluates the pipline based on the given dataset.
+        Evaluates the pipeline based on the given dataset.
 
         Args:
             model: the model given in the pipeline.
@@ -49,17 +77,18 @@ class ModelCallbackWrapper(Evaluator):
         """
         result = self.wrapped_evaluator.evaluate(model, data)
         fitted_model = self.wrapped_evaluator.fitted_model
-        self.callback(fitted_model, *self.callback_args,
-                      **self.callback_kwargs)
+        for callback in self.callbacks:
+            self.callback_results[callback.__name__] = callback(
+                fitted_model, *self.callback_args, **self.callback_kwargs)
         return result
 
     @property
     def configuration(self):
         """A json-like representation of the configuration."""
         return {
-            'model_callback': self.callback.__name__,
             'wrapped_evaluator': self.wrapped_evaluator.__class__.__name__,
             'wrapped_configuration': self.wrapped_evaluator.configuration,
+            'callbacks': self.callback_results,
         }
 
 
@@ -112,6 +141,12 @@ class GridEvaluator(Evaluator):
     @property
     def fitted_model(self):
         """Contains the filtered model."""
+        if self.model_ is None and \
+                'refit' in self.grid_parameters and \
+                self.grid_parameters['refit'] is False:
+            warnings.warn('you have set refit=False, but are trying to access'
+                          'the fitted model. It will always be None with this'
+                          'configuration.')
         return self.model_
 
     @property
@@ -168,10 +203,6 @@ class CustomCvGridEvaluator(GridEvaluator):
         """
         assert len(data) == 3
         x, y, cv = data
-
-        if not hasattr(cv, 'split'):
-            raise Exception(f'the submitted cv {cv} does not support split()')
-
         self.grid_parameters['cv'] = cv
         return super().evaluate(model, (x, y))
 
@@ -211,10 +242,7 @@ class FixedSplitGridEvaluator(GridEvaluator):
 
         Returns: A dict containting the results of the grid search.
         """
-        assert len(data) == 2
-        assert len(data[0]) == 2
-        assert len(data[1]) == 2
-        (xtrain, ytrain), (xtest, ytest) = data
+        (xtrain, ytrain), (xtest, ytest) = _extract_train_test_data(data)
         # index=-1 means that these samples will never be used for testing
         target_indices = [-1] * len(xtrain) + [0] * len(xtest)
         self.grid_parameters['cv'] = PredefinedSplit(target_indices)
@@ -231,13 +259,19 @@ class FixedSplitGridEvaluator(GridEvaluator):
 
         if type(xtrain) == np.ndarray:
             x = np.concatenate((xtrain, xtest))
-            y = np.concatenate((ytrain, ytest))
-        elif type(xtrain) == pd.DataFrame:
+        elif type(xtrain) in [pd.DataFrame, pd.Series]:
             x = pd.concat((xtrain, xtest))
+        else:
+            raise ValueError('don\'t know how to concatenate ', type(xtrain),
+                             type(xtest))
+
+        if type(ytrain) == np.ndarray:
+            y = np.concatenate((ytrain, ytest))
+        elif type(ytrain) in [pd.DataFrame, pd.Series]:
             y = pd.concat((ytrain, ytest))
         else:
-            raise ValueError(
-                f'cant concatenate train and test of type {type(xtrain)}')
+            raise ValueError('don\'t know how to concatenate ', type(ytrain),
+                             type(ytest))
 
         return super().evaluate(model, (x, y))
 
@@ -269,10 +303,7 @@ class FixedSplitEvaluator(Evaluator):
         Returns: A dict containting f1, accuracy, recall, precision and the
         confusion matrix.
         """
-        assert len(data) == 2
-        assert len(data[0]) == 2
-        assert len(data[1]) == 2
-        (xtrain, ytrain), (xtest, ytest) = data
+        (xtrain, ytrain), (xtest, ytest) = _extract_train_test_data(data)
         model.fit(xtrain, ytrain)
 
         self.model_ = model
@@ -366,10 +397,7 @@ class EpochEvaluator(FixedSplitEvaluator):
 
         Returns: A dict-like object containing the results.
         """
-        assert len(data) == 2
-        assert len(data[0]) == 2
-        assert len(data[1]) == 2
-        (xtrain, ytrain), (xtest, ytest) = data
+        (xtrain, ytrain), (xtest, ytest) = _extract_train_test_data(data)
 
         try:
             model_epochs = model.epochs
@@ -511,7 +539,7 @@ def _aggregate_score_dicts(scores):
     for epoch, result in scores[0].items():
         yield epoch, {
             key: np.asarray([score[epoch][key] for score in scores
-                             ]) for key in result.keys()  # noqa E124
+                            ]) for key in result.keys()  # noqa E124
         }
 
 
@@ -585,3 +613,25 @@ class DataframeEvaluatorWrapper(Evaluator):
             'wrapped_evaluator': self.wrapped_class.__name__,
             'wrapped_configuration': self.evaluator.configuration,
         }
+
+
+def _ensure_unique_file(name, append_timestamp=True):
+    if append_timestamp:
+        parts = os.path.splitext(name)
+        timestamp = datetime.datetime.now().strftime('%Y-%m-%d__%H-%M-%S')
+        name = f'{parts[0]}_{timestamp}{parts[1]}'
+    i = 0
+    while os.path.isfile(name):
+        parts = os.path.splitext(name)
+        name = f'{parts[0]}_{i}{parts[1]}'
+        i += 1
+    return name
+
+
+def pickle_model_callback(fitted_model, output_path):
+    """Pickles fitted models, for ModelCallbackEvaluator."""
+    unique_name = _ensure_unique_file(output_path)
+    with open(unique_name, 'wb') as output_f:
+        LOGGER.info('writing fitted model to %s', unique_name)
+        pickle.dump(fitted_model, output_f)
+        return unique_name
