@@ -2,6 +2,8 @@
 import pickle
 import tempfile
 
+from logzero import logger
+
 from .base import StorageHandler
 from .db import DB
 from .db import DbModel
@@ -48,16 +50,20 @@ class PickleBackupHandler(StorageHandler):
             result: passed from an evaluator.
         """
         # TODO: think of a useful naming/storage strategy
-        with tempfile.NamedTemporaryFile(
-                dir=self.directory, suffix='.result',
-                delete=False) as output_file:
+        with tempfile.NamedTemporaryFile(dir=self.directory,
+                                         suffix='.result',
+                                         delete=False) as output_file:
 
             pickle.dump(result, output_file)
             LOGGER.debug(f'wrote pickle to {output_file.name}')
 
 
 def _get_requirements():
-    return list(freeze.freeze())
+    try:
+        return list(freeze.freeze())
+    except OSError as ex:
+        logger.exception(ex)
+        return list()
 
 
 class PostGresOutputHandler(StorageHandler):
@@ -87,19 +93,18 @@ class PostGresOutputHandler(StorageHandler):
             project_name = self.config['project']['name']
         else:
             project_name = result['sourcefile']
-        db_entry = DbModel(
-            project_name=project_name,
-            git_commit_id=commit_id,
-            git_remote_url=remote_url,
-            git_is_dirty=is_dirty,
-            sourcefile=result['sourcefile'],
-            config=result['plan_content'],
-            dataloader=result['dataloader'],
-            pipeline=result['pipeline'],
-            evaluator=result['evaluator'],
-            outcome=result['outcome'],
-            platform=result['platform'],
-            requirements=_get_requirements())
+        db_entry = DbModel(project_name=project_name,
+                           git_commit_id=commit_id,
+                           git_remote_url=remote_url,
+                           git_is_dirty=is_dirty,
+                           sourcefile=result['sourcefile'],
+                           config=result['plan_content'],
+                           dataloader=result['dataloader'],
+                           pipeline=result['pipeline'],
+                           evaluator=result['evaluator'],
+                           outcome=result['outcome'],
+                           platform=result['platform'],
+                           requirements=_get_requirements())
 
         session = self.session()
 
